// Copyright 2002 Adam Megacz, see the COPYING file for licensing [GPL]
package org.xwt;

/**
 *  <p>
 *  A block of pixels which can be drawn on and rapidly copied to the
 *  screen. Drawing operations are performed on this class; it is
 *  then rendered to the screen with Surface.blit().
 *  </p>
 *
 *  <p>
 *  Implementations of the Platform class should return objects
 *  supporting this interface from the _createDoubleBuffer()
 *  method. These implementations may choose to use off-screen video
 *  ram for this purpose (for example, a Pixmap on X11).
 *  </p>
 *
 *  <p>
 *  A note on coordinates: all members on DoubleBuffer specify
 *  coordinates in terms of x1,y1,x2,y2 even though the Box class
 *  represents regions internally as x,y,w,h.
 *  </p>
 */
public interface DoubleBuffer {

    /** Draw the region of source within (sx1, sy1, sx2, sy2) onto the region of this DoubleBuffer within (dx1, dy1, dx2, dy2), scaling as needed. */
    public void drawPicture(Picture source, int dx1, int dy1, int dx2, int dy2, int sx1, int sy1, int sx2, int sy2);

    /** Draw source onto this DoubleBuffer at (x,y) */
    public void drawPicture(Picture source, int x, int y);

    /** Draw <tt>text</tt> in <tt>font</tt> and <tt>color</tt> on this DoubleBuffer, with the upper left corner of the text at (x, y) */
    public void drawString(String font, String text, int x, int y, int color);

    /** Fill the region (x1, y1, x2, y2) with <tt>color</tt> (AARRGGBB format); the alpha channel component is ignored */
    public void fillRect(int x1, int y1, int x2, int y2, int color);

    /** Sets the clip region for this DoubleBuffer to (x,y,x2,y2) */
    public void setClip(int x, int y, int x2, int y2);

    public int getHeight();
    public int getWidth();
}
